
package diagrams;

import java.awt.Graphics2D;
import java.awt.Rectangle;

/**
 *
 * @author B. Gaertner
 */
public class Grid extends AbstractGrid {

    HorizontalRenderableScale horizontalScale;
    VerticalRenderableScale verticalScale;
    boolean renderHorizontalGridLines;
    boolean renderVerticalGridLines;
    DiagramTransformation transformation;

    public Grid (HorizontalRenderableScale hs, VerticalRenderableScale vs) {
        horizontalScale = hs;
        verticalScale = vs;
        renderHorizontalGridLines = true;
        renderVerticalGridLines = true;
        transformation = new DiagramTransformation(horizontalScale, verticalScale);
    }

    @Override
    public int[] transform(double x, double y) {
        return transformation.transform(x, y);
    }

    @Override
    public void transform(double x, double y, int[] res) {
        transformation.transform(x, y, res);
    }

    @Override
    public void transformViewCoordinates(int x, int y, PositionRecorder p) {
        if (horizontalScale.includesImagePoint(x) && verticalScale.includesImagePoint(y)) {
            double tx = horizontalScale.transformIntoWorldCoordinate(x);
            double ty = verticalScale.transformIntoWorldCoordinate(y);
            p.setPosition(tx, ty);
        } else {
            p.clearPosition();
        }
    }

    @Override
    protected void renderGridOn(Graphics2D g) {
        if (this.renderHorizontalGridLines) {
            this.horizontalScale.renderGridLines(g, new VerticalPaintRange(verticalScale.box));
        }
        if (this.renderVerticalGridLines) {
            this.verticalScale.renderGridLines(g, new HorizontalPaintRange(horizontalScale.box));
        }
    }

    public void emphasizeAxes(Graphics2D g) {
        if (this.renderHorizontalGridLines) {
            this.horizontalScale.emphasizeAxis(g, new VerticalPaintRange(verticalScale.box));
        }
        if (this.renderVerticalGridLines) {
            this.verticalScale.emphasizeAxis(g, new HorizontalPaintRange(horizontalScale.box));
        }
    }

    public Rectangle clipRect() {
        return new Rectangle (horizontalScale.box.xStart(),
                verticalScale.box.yStart() - verticalScale.box.scaleHeight(),
                horizontalScale.box.scaleWidth() + 1,
                verticalScale.box.scaleHeight() + 1);
    }
}
