
package diagrams;


import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.util.Enumeration;

/**
 *
 * @author B. Gaertner
 */
public class HorizontalRenderableScale extends RenderableScale  {

    public HorizontalRenderableScale (AbstractScale s, ScaleBox b) {
        super (s, b);
    }

    @Override
    protected void renderScaleOn(Graphics2D g) {
        int x0 = this.box.xStart();
        int x1 = this.box.xEnd();
        int y1 = this.box.yEnd();
        int tickSize = this.scale.scaleProps.tickSize;

        FontMetrics fm = g.getFontMetrics(this.scale.scaleProps.scaleFont);
        ValueFormatter fmt = this.scale.scaleProps.formatter;

        g.drawLine(x0, y1, x1, y1);
        ScaleTransformation st = this.createTransformation(new HorizontalPaintRange(box));
        for(Enumeration<TickValue> e = scale.enumeration(); e.hasMoreElements();) {
            TickValue tv = e.nextElement();
            int tickCoordinate = st.transformToInt(tv.value);
            g.drawLine(tickCoordinate, y1, tickCoordinate, y1 + tickSize);
            if (scale.tickIsLabelled(tv)) {
                String labelString = fmt.format(tv.value);
                int w = fm.stringWidth(labelString);
                int xPos = tickCoordinate - w/2;
                int yPos = y1 + tickSize + fm.getAscent();
                g.drawString(labelString, xPos, yPos);
            }
        }
    }

    public void renderGridLines(Graphics2D g, AbstractPaintRange vBox) {
        ScaleTransformation st = this.createTransformation(new HorizontalPaintRange(box));
        int y0 = vBox.paintOrigin();
        int y1 = vBox.paintEndpoint();

        for(Enumeration<TickValue> e = scale.enumeration(); e.hasMoreElements();) {
            TickValue tv = e.nextElement();
            int tickCoordinate = st.transformToInt(tv.value);
            g.drawLine(tickCoordinate, y0, tickCoordinate, y1);
        }
    }

    public void emphasizeAxis(Graphics2D g, AbstractPaintRange vBox) {
        ScaleTransformation st = this.createTransformation(new HorizontalPaintRange(box));
        int y0 = vBox.paintOrigin();
        int y1 = vBox.paintEndpoint();

        if (hasAxisColor) {
            Color oldColor = g.getColor();
            g.setColor(axisColor);
            int tickCoordinate = st.transformToInt(0.0);
            g.drawLine(tickCoordinate, y0, tickCoordinate, y1);
            g.setColor(oldColor);
        }
    }

    public boolean includesImagePoint(int x) {
        return this.box.xStart() <= x && this.box.xEnd() >= x;
    }

    public double transformIntoWorldCoordinate(int x) {
        ScaleTransformation st = this.createTransformation(new HorizontalPaintRange(box));
        return st.inverseTransformation(x);
    }
}
