
package diagrams;

import java.util.Enumeration;

/**
 *
 * @author B. Gertner
 */
public class LinearScale extends AbstractScale {

    public LinearScale (AbstractScaleRange sr, ScaleProperties sp) {
        super(sr, sp);
    }

    @Override
    public ScaleTransformation createTransformation(AbstractPaintRange sb) {
        return new LinearTransformation
                (sb.paintWidth()/this.scaleRange.width(),
                this.scaleRange.startValue,
                sb.paintOrigin());
    }

    @Override
    public Enumeration<TickValue> enumeration() {
        return new LinearScaleTickValueEnumeration(this);
    }

    @Override
    public boolean tickIsLabelled(TickValue tv) {
        int count = tv.count;
        return (this.scaleRange.labelDistance == 1)
                || ((count + 1)%this.scaleRange.labelDistance == this.scaleRange.firstLabelledTick );
    }

    public double valueAtTick(int idx) {
        double firstValue = scaleRange.startValue;
        double lastValue = scaleRange.endValue;
        double delta = (lastValue - firstValue)/scaleRange.numberOfIntervals;
        return idx * delta + firstValue;
    }
}

class LinearScaleTickValueEnumeration implements Enumeration<TickValue> {

    double firstValue;
    double lastValue;
    double delta;
    int numberOfElements;
    int count;

    public LinearScaleTickValueEnumeration(LinearScale ls) {
        firstValue = ls.scaleRange.startValue;
        lastValue = ls.scaleRange.endValue;
        delta = (lastValue - firstValue)/ls.scaleRange.numberOfIntervals;
        numberOfElements = ls.scaleRange.numberOfIntervals;
        count = 0;
    }

    @Override
    public boolean hasMoreElements() {
        return count < numberOfElements + 1;
    }

    @Override
    public TickValue nextElement() {
        TickValue tv = new TickValue(0, count, delta*count + firstValue);
        count += 1;
        return tv;
    }
}
