
package diagrams;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.util.Enumeration;

/**
 *
 * @author B. Gaertner
 */
public class VerticalRenderableScale extends RenderableScale {

    public VerticalRenderableScale (AbstractScale s, ScaleBox b) {
        super(s, b);
    }

    @Override
    protected void renderScaleOn(Graphics2D g) {
        int y0 = this.box.yStart();
        int x1 = this.box.xEnd();
        int y1 = this.box.yEnd();
        int tickSize = this.scale.scaleProps.tickSize;

        FontMetrics fm = g.getFontMetrics(this.scale.scaleProps.scaleFont);
        ValueFormatter fmt = this.scale.scaleProps.formatter;

        g.drawLine(x1, y0, x1, y1);

        ScaleTransformation st = this.createTransformation(new VerticalPaintRange(box));
        for(Enumeration<TickValue> e = scale.enumeration(); e.hasMoreElements();) {
            TickValue tv = e.nextElement();
            int tickCoordinate = st.transformToInt(tv.value);
            g.drawLine(x1 - tickSize, tickCoordinate, x1, tickCoordinate);
            if (scale.tickIsLabelled(tv)) {
                String labelString = fmt.format(tv.value);
                int w = fm.stringWidth(labelString);
                int xPos = x1 - tickSize - w - 2;
                int yPos = tickCoordinate + fm.getAscent()/2 - fm.getDescent() / 2;
                g.drawString(labelString, xPos, yPos);
            }
        }
    }

    public void renderGridLines(Graphics2D g, AbstractPaintRange hBox) {
        ScaleTransformation st = this.createTransformation(new VerticalPaintRange(box));
        int x0 = hBox.paintOrigin();
        int x1 = hBox.paintEndpoint();

        for(Enumeration<TickValue> e = scale.enumeration(); e.hasMoreElements();) {
            TickValue tv = e.nextElement();
            int tickCoordinate = st.transformToInt(tv.value);
            g.drawLine(x0, tickCoordinate, x1, tickCoordinate);
        }
    }

    public void emphasizeAxis(Graphics2D g, AbstractPaintRange hBox) {
        ScaleTransformation st = this.createTransformation(new VerticalPaintRange(box));
        int x0 = hBox.paintOrigin();
        int x1 = hBox.paintEndpoint();
        if (hasAxisColor) {
            Color oldColor = g.getColor();
            g.setColor(axisColor);
            int tickCoordinate = st.transformToInt(0.0);
            g.drawLine(x0, tickCoordinate, x1, tickCoordinate);
            g.setColor(oldColor);
        }
    }

    public boolean includesImagePoint(int y) {
        return this.box.yStart() >= y && this.box.yEnd() <= y;
    }

    public double transformIntoWorldCoordinate(int x) {
        ScaleTransformation st = this.createTransformation(new VerticalPaintRange(box));
        return st.inverseTransformation(x);
    }
}
