
package window;

import diagrams.AbstractGrid;
import diagrams.DecimalLabelFormat;
import diagrams.Grid;
import diagrams.HorizontalRenderableScale;
import diagrams.LinearScale;
import diagrams.LinearScaleRange;
import diagrams.ScaleBox;
import diagrams.ScaleProperties;
import diagrams.ScaledLabelFormat;
import diagrams.ValueFormatter;
import diagrams.VerticalRenderableScale;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Stroke;
import static java.lang.Math.PI;
import java.util.Observable;
import java.util.Observer;
import javax.swing.JComponent;

public class DrawingPanel extends JComponent implements Observer {

    ApplicationModel model;

    public DrawingPanel (ApplicationModel m) {
        model = m;
        this.registerAsObserver();
    }

    private void registerAsObserver() {
        model.addObserver(this);
    }

    @Override
    public void paintComponent(Graphics arg) {
        super.paintComponent(arg);
        Dimension groesse = this.getSize();
        Insets insets = this.getInsets();

        java.awt.Rectangle bereich = new java.awt.Rectangle(insets.left, insets.top,
                groesse.width - (insets.left + insets.right),
                groesse.height - (insets.top + insets.bottom));
        Graphics2D g = (Graphics2D) arg;

        java.awt.Stroke st = g.getStroke();   //  Strichbreite speichern
        this.InhaltZeichnen(g, bereich);
        g.setStroke(st);
    }

    public void InhaltZeichnen(Graphics2D g, Rectangle bereich) {
        g.setColor(Color.WHITE);
        java.awt.Rectangle r = (java.awt.Rectangle) bereich.clone();
        g.fillRect(r.x, r.y,
                r.width,
                r.height);    // Hintergrund zeichnen
        g.setColor(Color.LIGHT_GRAY);
        this.paintDiagram(g, r);
    }

    @Override
    public void update(Observable o, Object arg) {
        this.repaint();
    }

    private void paintDiagram(Graphics2D gr, Rectangle rect) {
        boolean suppressScales = rect.width < 300 || rect.height < 300;
        int topSpace = suppressScales ? 10 : 15;
        int rightSpace = suppressScales ? 10 : 25;
        int vScaleWidth = suppressScales ? 10 : 65;
        int hScaleHeight = suppressScales ? 10 : 40;

        LinearScaleRange sr1 = new LinearScaleRange(-8.0*PI, 8.0*PI, 16, 2, 1);
        LinearScaleRange sr2 = new LinearScaleRange(-10.0, 10.0, 10, 2);

        //  aspect ratio ist 3*PI
        Font fnt = new Font("Times", Font.PLAIN, 18);
        ValueFormatter fmt2 = new DecimalLabelFormat("#.0");
        ScaleProperties sp2 = new ScaleProperties(Color.GRAY, fnt, fmt2);
         //  U+03C0  ist pi
        ScaledLabelFormat fmt1 = new ScaledLabelFormat(PI, "\u03c0");
        ScaleProperties sp1 = new ScaleProperties(Color.GRAY, fnt, fmt1);

        ScaleBox hBox;
        ScaleBox vBox;
        HorizontalRenderableScale hScale =
                new HorizontalRenderableScale(
                        new LinearScale(sr1, sp1),
                        hBox = new ScaleBox(new Rectangle(rect.x, rect.y + rect.height - hScaleHeight,
                                                   rect.width, hScaleHeight),
                                     new Insets(3, vScaleWidth, 3, rightSpace)
                       ));
        VerticalRenderableScale vScale =
                new VerticalRenderableScale(
                        new LinearScale(sr2, sp2),
                        vBox = new ScaleBox(new Rectangle(rect.x, rect.y,
                                                   vScaleWidth, rect.height),
                                     new Insets(topSpace, 3, hScaleHeight, 3)
                       ));
        //System.out.println("width = " + hBox.scaleWidth());
        //System.out.println("height = " + vBox.scaleHeight());
        //double windowRatio = hBox.scaleWidth() / vBox.scaleHeight();

        /*if (windowRatio < 3*PI){
            System.out.println("vertical excess");
        } else {
            System.out.println("horizontal excess");
        } */

        Grid grid = new Grid(hScale, vScale);
        hScale.axisColor(Color.BLACK);
        vScale.axisColor(Color.BLACK);

        if (! suppressScales) {
            hScale.renderOn(gr);
            vScale.renderOn(gr);
        }
        //  hier wre der Hintergrund zu zeichnen.
        grid.renderOn(gr);
        grid.emphasizeAxes(gr);
        gr.setClip(grid.clipRect());
        this.paintOnGrid(grid, gr);
    }

    public void paintOnGrid(AbstractGrid grid, Graphics2D g) {
        double a = model.getAmplitude();
        double b = model.getPeriode();
        double deltaX = model.getDeltaX();
        double deltaY = model.getDeltaY();
        int nroOfPoints;
        if (b != 0)
        {
            float nrP = Math.abs((float)b)*8.0f;
            //System.out.println("Anzahl perioden = " + Double.toString(nrP));
            nroOfPoints = 50*Math.round(nrP);
        } else {
            nroOfPoints = 20;
        }

        int xPoints [] = new int[nroOfPoints];
        int yPoints [] = new int[nroOfPoints];
        int buffer [] = new int[2];
        RenderingHints einstellungen = new RenderingHints(
            RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
        g.addRenderingHints(einstellungen);

        g.setStroke(new BasicStroke(3, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));

                //  Hier deltaX, deltaY farbig einzeichnen:
        if (deltaX != 0) {
            g.setColor(new Color(0, 128, 0));

            grid.transform(0.0, 0.0, buffer);
            xPoints[0] = buffer[0];
            yPoints[0] = buffer[1];

            grid.transform(-deltaX, 0.0, buffer);
            xPoints[1] = buffer[0];
            yPoints[1] = buffer[1];
            g.drawPolyline(xPoints, yPoints, 2);
        }
        if (deltaY != 0) {
            g.setColor(Color.MAGENTA);

            grid.transform(-deltaX, 0.0, buffer);
            xPoints[0] = buffer[0];
            yPoints[0] = buffer[1];

            grid.transform(-deltaX, deltaY, buffer);
            xPoints[1] = buffer[0];
            yPoints[1] = buffer[1];
            g.drawPolyline(xPoints, yPoints, 2);
        }


        for (int i = 0; i < nroOfPoints; i++) {
            double x = 16.0*PI*i/(nroOfPoints -1) - 8.0*PI;
            double y = (model.getSelection() == 1
                           ? Math.sin(b*(x + deltaX))
                           : Math.cos(b*(x + deltaX)))*a + deltaY;
            grid.transform(x, y, buffer);

            xPoints[i] = buffer[0];
            yPoints[i] = buffer[1];
        }
        Stroke oldStroke = g.getStroke();
        g.setColor(Color.red);
        g.setStroke(new BasicStroke(3, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
        g.drawPolyline(xPoints, yPoints, nroOfPoints);
        g.setStroke(oldStroke);
    }

}