
package window;

import diagrams.DecimalLabelFormat;
import diagrams.ValueFormatter;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JToggleButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.SwingConstants;

/**
 *
 * @author B. Gaertner
 */
public class MainWindow extends JFrame
           implements Runnable, LayoutManager {


    ApplicationModel model;
    /**
     * The entry point of the program.
     */
    @Override
    public void run() {
        model = new ApplicationModel();
        this.createGUI();
    }

     /**
     * Set up the components of the main window and make the window visible.
     */
    private void createGUI() {
        this.setTitle("sin - cos");
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        this.addComponentsToPane(this.getContentPane());
        pack();
        this.setSize(760, 600);
        this.setLocationRelativeTo(null); // center on screen
        this.setVisible(true);
    }

    /**
     * The method creates the visual elements that comprise the window.
     * @param container
     */
    private void addComponentsToPane (Container container) {
        container.setLayout(this);
        ButtonGroup buttonGroup = new ButtonGroup();
        JToggleButton bttnSin = new JToggleButton("sin");
        JToggleButton bttnCos = new JToggleButton("cos");
        bttnSin.setSelected(model.getSelection() == 1);
        bttnCos.setSelected(model.getSelection() == 2);

        JCheckBox check = new JCheckBox("Grundfunktion anzeigen");
        check.setSelected(model.getShowNormalFn());

        JLabel label1 = new JLabel("Amplitude");
        JLabel label2 = new JLabel("Periode");
        JLabel label3 = new JLabel("x-Verschiebung");
        JLabel label4 = new JLabel("y-Verschiebung");
        label1.setForeground(Color.red);
        label2.setForeground(Color.BLUE);
        label3.setForeground(new Color(0, 128, 0));
        label4.setForeground(Color.MAGENTA);
        JSlider sliderAmplitude = new JSlider(model.getAmplitudeModel());
        JSlider sliderPeriode = new JSlider(model.getPeriodeModel());
        JSlider sliderXTranslation = new JSlider(model.getTranslationXModel());
        JSlider sliderYTranslation = new JSlider(model.getTranslationYModel());
        sliderAmplitude.setOrientation(SwingConstants.HORIZONTAL);
        sliderPeriode.setOrientation(SwingConstants.HORIZONTAL);
        sliderXTranslation.setOrientation(SwingConstants.HORIZONTAL);
        sliderXTranslation.setOrientation(SwingConstants.HORIZONTAL);
        sliderAmplitude.addChangeListener(model);
        sliderPeriode.addChangeListener(model);
        sliderXTranslation.addChangeListener(model);
        sliderYTranslation.addChangeListener(model);

        ValueFormatter fmt = new DecimalLabelFormat("#0.00");
        ValueDisplayField valueField1 = new ValueDisplayField(model, fmt, KindOfValue.AMPLITUDE);
        ValueDisplayField valueField2 = new ValueDisplayField(model, new PeriodeFormat("#0.00"), KindOfValue.PERIODE);
        ValueDisplayField valueField3 = new ValueDisplayField(model, fmt, KindOfValue.DELTA_X);
        ValueDisplayField valueField4 = new ValueDisplayField(model, fmt, KindOfValue.DELTA_Y);
        valueField1.setForeground(Color.red);
        valueField2.setForeground(Color.BLUE);
        valueField3.setForeground(new Color(0, 128, 0));
        valueField4.setForeground(Color.MAGENTA);

        DrawingPanel paintArea = new DrawingPanel(model);
        FormulaDisplayField formula = new FormulaDisplayField(model);

        buttonGroup.add(bttnSin);
        buttonGroup.add(bttnCos);
        bttnSin.addActionListener(new ButtonChangeAdapter(model, 1));
        bttnCos.addActionListener(new ButtonChangeAdapter(model, 2));
        check.addItemListener(new CheckBoxItemListener(model));

        container.add(bttnSin);
        container.add(bttnCos);
        container.add(check);
        container.add(label1);
        container.add(sliderAmplitude);
        container.add(valueField1);
        container.add(label2);
        container.add(sliderPeriode);
        container.add(valueField2);
        container.add(label3);
        container.add(sliderXTranslation);
        container.add(valueField3);
        container.add(label4);
        container.add(sliderYTranslation);
        container.add(valueField4);
        container.add(formula);
        container.add(paintArea);
    }

    @Override
    public void addLayoutComponent(String name, Component comp) {
        //  no action needed
    }

    @Override
    public void removeLayoutComponent(Component comp) {
        //  no action needed
    }

    @Override
    public Dimension preferredLayoutSize(Container parent) {
        Dimension dim = new Dimension(0, 0);

        //Always add the container's insets!
        Insets insets = parent.getInsets();
        dim.width = 100 + insets.left + insets.right;
        dim.height = 100 + insets.top + insets.bottom;
        return dim;
    }

    @Override
    public Dimension minimumLayoutSize(Container parent) {
        Dimension dim = new Dimension(0, 0);
        //Always add the container's insets!
        Insets insets = parent.getInsets();
        dim.width = 60 + insets.left + insets.right;
        dim.height = 26 + insets.top + insets.bottom;
        return dim;
    }

    @Override
    public void layoutContainer(Container parent) {
        Insets insets = parent.getInsets();
        final int maxWidth = parent.getWidth() - (insets.left + insets.right);
        final int maxHeight = parent.getHeight() - (insets.top + insets.bottom);
        final int x0 = insets.left;
        final int y0 = insets.top;
        final int buttonHeight = 26;
        final int sliderHeight = 30;
        final int labelWidth = 100;
        final int valueFieldWidth = 80;
        final int expressionFieldHeight = 80;
        final int widthButton;
        final int widthButtonField;
        final int widthCheckBox;
        Component c;
        widthButton = maxWidth / 4;
        widthButtonField = widthButton * 2;
        widthCheckBox = maxWidth - widthButtonField;

        c = parent.getComponent(0);
        c.setBounds(x0, y0, widthButton, buttonHeight);
        c = parent.getComponent(1);
        c.setBounds(x0 + widthButton, y0, widthButton, buttonHeight);
        c = parent.getComponent(2);   //  checkBox
        c.setBounds(x0 + widthButtonField, y0, widthCheckBox, buttonHeight);

        c = parent.getComponent(3);    //  label 1
        c.setBounds(x0 + 5, y0 + maxHeight - 4*sliderHeight, labelWidth - 5, sliderHeight);
        c = parent.getComponent(4);   // Slider 1
        c.setBounds(x0 + labelWidth + valueFieldWidth, y0 + maxHeight - 4*sliderHeight,
                    maxWidth - labelWidth - valueFieldWidth, sliderHeight);
        c = parent.getComponent(5);   //  wertfeld
        c.setBounds(x0 + labelWidth, y0 + maxHeight - 4*sliderHeight, valueFieldWidth, sliderHeight);
        c = parent.getComponent(6);  // label 2
        c.setBounds(x0 + 5, y0 + maxHeight - 3*sliderHeight, labelWidth - 5, sliderHeight);
        c = parent.getComponent(7);   // Slider 2
        c.setBounds(x0 + labelWidth + valueFieldWidth, y0 + maxHeight - 3*sliderHeight,
                    maxWidth - labelWidth - valueFieldWidth, sliderHeight);
        c = parent.getComponent(8);   //  Wertfeld
        c.setBounds(x0 + labelWidth, y0 + maxHeight - 3*sliderHeight, valueFieldWidth, sliderHeight);

        c = parent.getComponent(9);    //  label 3
        c.setBounds(x0 + 5, y0 + maxHeight - 2*sliderHeight, labelWidth - 5, sliderHeight);
        c = parent.getComponent(10);   // Slider 3
        c.setBounds(x0 + labelWidth + valueFieldWidth, y0 + maxHeight - 2*sliderHeight,
                    maxWidth - labelWidth - valueFieldWidth, sliderHeight);
        c = parent.getComponent(11);   //  Wertfeld
        c.setBounds(x0 + labelWidth, y0 + maxHeight - 2*sliderHeight, valueFieldWidth, sliderHeight);
        c = parent.getComponent(12);  // label 4
        c.setBounds(x0 + 5, y0 + maxHeight - 1*sliderHeight, labelWidth - 5, sliderHeight);
        c = parent.getComponent(13);   // Slider 4
        c.setBounds(x0 + labelWidth + valueFieldWidth, y0 + maxHeight - 1*sliderHeight,
                    maxWidth - labelWidth - valueFieldWidth, sliderHeight);
        c = parent.getComponent(14);   // Wertfeld
        c.setBounds(x0 + labelWidth, y0 + maxHeight - 1*sliderHeight, valueFieldWidth, sliderHeight);
        c = parent.getComponent(15);   //  formelanzeige
        c.setBounds(x0, y0 + maxHeight - 4*sliderHeight - expressionFieldHeight, maxWidth, expressionFieldHeight);
        c = parent.getComponent(16);   //  paint area
        c.setBounds(x0, y0 + buttonHeight, maxWidth, maxHeight - buttonHeight - 4*sliderHeight - expressionFieldHeight);
    }
}
